@@ -53,15 +53,11 @@ struct _GtkhtmlFaceToolButtonPrivate {
 
 	guint popup_shown	: 1;
 	guint popup_in_progress	: 1;
-	GdkDevice *grab_keyboard;
-	GdkDevice *grab_mouse;
 };
 
 static gpointer parent_class;
 static guint signals[LAST_SIGNAL];
 
-static void face_tool_button_build_popup_window (GtkhtmlFaceToolButton *button);
-
 /* XXX Copied from _gtk_toolbar_elide_underscores() */
 static gchar *
 face_tool_button_elide_underscores (const gchar *original)
@@ -275,13 +271,8 @@ face_tool_button_dispose (GObject *object)
 
 	priv = GTKHTML_FACE_TOOL_BUTTON (object)->priv;
 
-	if (priv->table != NULL) {
-		g_object_unref (priv->table);
-		priv->table = NULL;
-	}
-
 	if (priv->window != NULL) {
-		gtk_widget_destroy (priv->window);
+		g_object_unref (priv->window);
 		priv->window = NULL;
 	}
 
@@ -334,12 +325,7 @@ face_tool_button_popup (GtkhtmlFaceToolButton *button)
 {
 	GtkToggleToolButton *tool_button;
 	GdkWindow *window;
-	gboolean grab_status;
-	GdkDevice *device, *mouse, *keyboard;
-	guint32 activate_time;
-
-	device = gtk_get_current_event_device ();
-	g_return_if_fail (device != NULL);
+	GdkGrabStatus status;
 
 	if (!gtk_widget_get_realized (GTK_WIDGET (button)))
 		return;
@@ -347,18 +333,6 @@ face_tool_button_popup (GtkhtmlFaceToolButton *button)
 	if (button->priv->popup_shown)
 		return;
 
-	if (!button->priv->window)
-		face_tool_button_build_popup_window (button);
-
-	activate_time = gtk_get_current_event_time ();
-	if (gdk_device_get_source (device) == GDK_SOURCE_KEYBOARD) {
-		keyboard = device;
-		mouse = gdk_device_get_associated_device (device);
-	} else {
-		keyboard = gdk_device_get_associated_device (device);
-		mouse = device;
-	}
-
 	/* Position the window over the button. */
 	face_tool_button_reposition_window (button);
 
@@ -372,28 +346,22 @@ face_tool_button_popup (GtkhtmlFaceToolButton *button)
 
 	/* Try to grab the pointer and keyboard. */
 	window = gtk_widget_get_window (button->priv->window);
-	grab_status = !keyboard ||
-		gdk_device_grab (keyboard, window,
-			GDK_OWNERSHIP_WINDOW, TRUE,
-			GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK,
-			NULL, activate_time) == GDK_GRAB_SUCCESS;
-	if (grab_status) {
-		grab_status = !mouse ||
-			gdk_device_grab (mouse, window,
-				GDK_OWNERSHIP_WINDOW, TRUE,
-				GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK,
-				NULL, activate_time) == GDK_GRAB_SUCCESS;
-		if (!grab_status && keyboard)
-			gdk_device_ungrab (keyboard, activate_time);
+	status = gdk_pointer_grab (
+		window, TRUE,
+		GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+		GDK_POINTER_MOTION_MASK, NULL, NULL, GDK_CURRENT_TIME);
+	if (status == GDK_GRAB_SUCCESS) {
+		status = gdk_keyboard_grab (window, TRUE, GDK_CURRENT_TIME);
+		if (status != GDK_GRAB_SUCCESS)
+			gdk_display_pointer_ungrab (
+				gdk_window_get_display (window),
+				GDK_CURRENT_TIME);
 	}
 
-	if (grab_status) {
-		gtk_device_grab_add (button->priv->window, mouse, TRUE);
-		button->priv->grab_keyboard = keyboard;
-		button->priv->grab_mouse = mouse;
-	} else {
+	if (status == GDK_GRAB_SUCCESS)
+		gtk_grab_add (button->priv->window);
+	else
 		gtk_widget_hide (button->priv->window);
-	}
 }
 
 static void
@@ -408,20 +376,12 @@ face_tool_button_popdown (GtkhtmlFaceToolButton *button)
 		return;
 
 	/* Hide the pop-up. */
-	gtk_device_grab_remove (button->priv->window, button->priv->grab_mouse);
+	gtk_grab_remove (button->priv->window);
 	gtk_widget_hide (button->priv->window);
 
 	/* Deactivate the tool button. */
 	tool_button = GTK_TOGGLE_TOOL_BUTTON (button);
 	gtk_toggle_tool_button_set_active (tool_button, FALSE);
-
-	if (button->priv->grab_keyboard)
-		gdk_device_ungrab (button->priv->grab_keyboard, GDK_CURRENT_TIME);
-	if (button->priv->grab_mouse)
-		gdk_device_ungrab (button->priv->grab_mouse, GDK_CURRENT_TIME);
-
-	button->priv->grab_keyboard = NULL;
-	button->priv->grab_mouse = NULL;
 }
 
 static GtkhtmlFace *
@@ -547,14 +507,6 @@ face_tool_button_iface_init (GtkhtmlFaceChooserIface *iface)
 
 static void
 face_tool_button_init (GtkhtmlFaceToolButton *button)
-{
-	button->priv = G_TYPE_INSTANCE_GET_PRIVATE (
-		button, GTKHTML_TYPE_FACE_TOOL_BUTTON,
-		GtkhtmlFaceToolButtonPrivate);
-}
-
-static void
-face_tool_button_build_popup_window (GtkhtmlFaceToolButton *button)
 {
 	GtkhtmlFaceChooser *chooser;
 	GtkWidget *toplevel;
@@ -564,21 +516,25 @@ face_tool_button_build_popup_window (GtkhtmlFaceToolButton *button)
 	GList *list, *iter;
 	gint ii;
 
-	g_return_if_fail (button->priv->window == NULL);
+	button->priv = G_TYPE_INSTANCE_GET_PRIVATE (
+		button, GTKHTML_TYPE_FACE_TOOL_BUTTON,
+		GtkhtmlFaceToolButtonPrivate);
+
+	/* Build the pop-up window. */
 
 	window = gtk_window_new (GTK_WINDOW_POPUP);
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (button));
 	gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
 	gtk_window_set_type_hint (
 		GTK_WINDOW (window), GDK_WINDOW_TYPE_HINT_COMBO);
-	if (GTK_IS_WINDOW (toplevel)) {
+	if (gtk_widget_is_toplevel (toplevel)) {
 		gtk_window_group_add_window (
 			gtk_window_get_group (GTK_WINDOW (toplevel)),
 			GTK_WINDOW (window));
 		gtk_window_set_transient_for (
 			GTK_WINDOW (window), GTK_WINDOW (toplevel));
 	}
-	button->priv->window = window;
+	button->priv->window = g_object_ref (window);
 
 	g_signal_connect_swapped (
 		window, "show",
@@ -624,7 +580,7 @@ face_tool_button_build_popup_window (GtkhtmlFaceToolButton *button)
 		gchar *tooltip;
 
 		tooltip = face_tool_button_elide_underscores (
-			_(face->label));
+			gettext (face->label));
 
 		widget = gtk_button_new ();
 		gtk_button_set_image (
